% draw Wigner function of some selected quantum states

%%%% numerical details for position space representation

Npts = 1024;	 % number of points
L = 10;			 % half length of domain (will extend from -L to L)
dx = 2*L/Npts;
Pmax =  pi/dx;      %length in momentum space (hbar=1)
dp = 2*Pmax/Npts;   

%%%% Assemble grids

xgrid = linspace(-L,L-dx,Npts);
pgrid = linspace(-Pmax,Pmax-dp,Npts);
[X Y] = ndgrid(xgrid,xgrid);

XX = X + Y/2;
XXprime = X - Y/2;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% 
% Choice of quantum state. Comment/Uncomment as needed
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

choosestate = 3;

if(choosestate==0)
  %%% Implement your own choice here. See examples below. Make sure all three functions entered (Psi_x,Psi_XX,Psi_XXPrime) are identical
  %   and use .* (componentwise multiplication) to multiply any factors in your function that depend on x,XX,XXPrime


end
if(choosestate==1)
  %(1) Superposition of Gaussians (Example 29 of the lecture)
  a = -4;
  b = 4;
  sigma = 0.8;

  normfact = 1/(pi*sigma*sigma)^(1/4);
  Psi_x = normfact*(exp(-(xgrid-a).*(xgrid-a)/2/sigma/sigma) + exp(-(xgrid-b).*(xgrid-b)/2/sigma/sigma)); 
  Psi_XX = normfact*(exp(-(XX-a).*(XX-a)/2/sigma/sigma) + exp(-(XX-b).*(XX-b)/2/sigma/sigma));
  Psi_XXPrime = normfact*(exp(-(XXprime-a).*(XXprime-a)/2/sigma/sigma) + exp(-(XXprime-b).*(XXprime-b)/2/sigma/sigma));
end
if(choosestate==2)
  %(2) moving Gaussian wavepacket
  a = -3;
  p = 4; %\hbar=1, hence p=k
  sigma = 1.5;

  normfact = 1/(pi*sigma*sigma)^(1/4);
  Psi_x = normfact*exp(-(xgrid-a).*(xgrid-a)/2/sigma/sigma).*exp(i*p*xgrid); 
  Psi_XX = normfact*exp(-(XX-a).*(XX-a)/2/sigma/sigma).*exp(i*p*XX); 
  Psi_XXPrime = normfact*exp(-(XXprime-a).*(XXprime-a)/2/sigma/sigma).*exp(i*p*XXprime);  

end

if(choosestate==3)
  %(3) States expressed in terms of harmonic oscillator eigenstates
   nosci1 = 5;

   normfact = sqrt(1/(2^nosci1*factorial(nosci1)))*(1/pi)^0.25;
   sigma = 1.0;

   Psi_x  = normfact*exp(-xgrid.*xgrid/2.0/sigma/sigma).*polyval(HermitePoly(nosci1),xgrid/sigma);
   Psi_XX =normfact*exp(-XX.*XX/2.0/sigma/sigma).*polyval(HermitePoly(nosci1),XX/sigma);
   Psi_XXPrime = normfact*exp(-XXprime.*XXprime/2.0/sigma/sigma).*polyval(HermitePoly(nosci1),XXprime/sigma); 

end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Do the Fourier transforms

rho =  conj(Psi_XX).*Psi_XXPrime;
Wigner = ifftshift(ifft(fftshift(rho,2),[],2),2);    %%% POSSIBLE ALLOCATION SCREWUP, conj should have + ?fix lecture notes too
Psi_p = fftshift(fft(ifftshift(Psi_x)));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%plotting, you may have to adjust xrange and prange depending on your choice of state

xrange = [-6 6];
prange = [-6 6];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

subplot(1,3,1)
plot(xgrid,abs(Psi_x.*Psi_x),'k');
axis tight
xlabel(['x'])
ylabel(['n'])
title(['Position space density'])

subplot(1,3,2)
mypcolor(xgrid,pgrid,transpose(real(Wigner))/max(real(Wigner(:))));
axis tight
daspect([1 1 1])
xlim(xrange)
ylim(prange)
colorbar
xlabel(['x'])
ylabel(['p'])
title(['Wigner function'])

subplot(1,3,3)
plot(pgrid,abs(conj(Psi_p).*Psi_p),'k');
axis tight
xlim(prange)
xlabel(['p'])
ylabel(['n'])
title(['Momentum space density'])


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lw=2;
fs=20;

lineobj = findobj('type', 'Line');
set(lineobj,'LineWidth', lw);

patchobj = findobj('type', 'Patch');
set(patchobj,'LineWidth', lw);

textobj = findobj('type', 'axes');
set(textobj,'LineWidth', lw);
set(textobj, 'fontsize', fs);
set(textobj, 'FontName', 'Times');
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%  
